@@ -5,7 +5,8 @@ from datetime import datetime
 import hashlib
 import pathlib
 import sqlite_utils
-from sqlite_utils.db import AlterError, BadMultiValues, DescIndex
+from sqlite_utils.db import AlterError, BadMultiValues, DescIndex, NoTable
+from sqlite_utils.utils import maximize_csv_field_size_limit
 from sqlite_utils import recipes
 import textwrap
 import inspect
@@ -17,6 +18,7 @@ import sys
 import csv as csv_std
 import tabulate
 from .utils import (
+    OperationalError,
     _compile_code,
     chunks,
     file_progress,
@@ -46,17 +48,7 @@ If you do not know the encoding, running 'file filename.csv' may tell you.
 It's often worth trying: --encoding=latin-1
 """.strip()
 
-
-# Increase CSV field size limit to maximum possible
-# https://stackoverflow.com/a/15063941
-field_size_limit = sys.maxsize
-
-while True:
-    try:
-        csv_std.field_size_limit(field_size_limit)
-        break
-    except OverflowError:
-        field_size_limit = int(field_size_limit / 10)
+maximize_csv_field_size_limit()
 
 
 def output_options(fn):
@@ -102,7 +94,7 @@ def load_extension_option(fn):
     return click.option(
         "--load-extension",
         multiple=True,
-        help="SQLite extensions to load",
+        help="Path to SQLite extension, with optional :entrypoint",
     )(fn)
 
 
@@ -354,7 +346,7 @@ def analyze(path, names):
                 db.analyze(name)
         else:
             db.analyze()
-    except sqlite3.OperationalError as e:
+    except OperationalError as e:
         raise click.ClickException(e)
 
 
@@ -426,9 +418,22 @@ def dump(path, load_extension):
     required=False,
     help="Add NOT NULL DEFAULT 'TEXT' constraint",
 )
+@click.option(
+    "--ignore",
+    is_flag=True,
+    help="If column already exists, do nothing",
+)
 @load_extension_option
 def add_column(
-    path, table, col_name, col_type, fk, fk_col, not_null_default, load_extension
+    path,
+    table,
+    col_name,
+    col_type,
+    fk,
+    fk_col,
+    not_null_default,
+    ignore,
+    load_extension,
 ):
     """Add a column to the specified table
 
@@ -439,9 +444,13 @@ def add_column(
     """
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
-    db[table].add_column(
-        col_name, col_type, fk=fk, fk_col=fk_col, not_null_default=not_null_default
-    )
+    try:
+        db[table].add_column(
+            col_name, col_type, fk=fk, fk_col=fk_col, not_null_default=not_null_default
+        )
+    except OperationalError as ex:
+        if not ignore:
+            raise click.ClickException(str(ex))
 
 
 @cli.command(name="add-foreign-key")
@@ -547,6 +556,7 @@ def index_foreign_keys(path, load_extension):
 @click.option("--unique", help="Make this a unique index", default=False, is_flag=True)
 @click.option(
     "--if-not-exists",
+    "--ignore",
     help="Ignore if index already exists",
     default=False,
     is_flag=True,
@@ -607,9 +617,14 @@ def create_index(
     default=False,
     is_flag=True,
 )
+@click.option(
+    "--replace",
+    is_flag=True,
+    help="Replace existing FTS configuration if it exists",
+)
 @load_extension_option
 def enable_fts(
-    path, table, column, fts4, fts5, tokenize, create_triggers, load_extension
+    path, table, column, fts4, fts5, tokenize, create_triggers, replace, load_extension
 ):
     """Enable full-text search for specific table and columns"
 
@@ -627,12 +642,16 @@ def enable_fts(
 
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
-    db[table].enable_fts(
-        column,
-        fts_version=fts_version,
-        tokenize=tokenize,
-        create_triggers=create_triggers,
-    )
+    try:
+        db[table].enable_fts(
+            column,
+            fts_version=fts_version,
+            tokenize=tokenize,
+            create_triggers=create_triggers,
+            replace=replace,
+        )
+    except OperationalError as ex:
+        raise click.ClickException(ex)
 
 
 @cli.command(name="populate-fts")
@@ -907,9 +926,12 @@ def insert_upsert_implementation(
     load_extension=None,
     silent=False,
     bulk_sql=None,
+    functions=None,
 ):
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
+    if functions:
+        _register_functions(db, functions)
     if (delimiter or quotechar or sniff or no_headers) and not tsv:
         csv = True
     if (nl + csv + tsv) >= 2:
@@ -1033,7 +1055,7 @@ def insert_upsert_implementation(
         )
     except Exception as e:
         if (
-            isinstance(e, sqlite3.OperationalError)
+            isinstance(e, OperationalError)
             and e.args
             and "has no column named" in e.args[0]
         ):
@@ -1286,6 +1308,9 @@ def upsert(
 @click.argument("sql")
 @click.argument("file", type=click.File("rb"), required=True)
 @click.option("--batch-size", type=int, default=100, help="Commit every X records")
+@click.option(
+    "--functions", help="Python code defining one or more custom SQL functions"
+)
 @import_options
 @load_extension_option
 def bulk(
@@ -1293,6 +1318,7 @@ def bulk(
     sql,
     file,
     batch_size,
+    functions,
     flatten,
     nl,
     csv,
@@ -1349,8 +1375,9 @@ def bulk(
             load_extension=load_extension,
             silent=False,
             bulk_sql=sql,
+            functions=functions,
         )
-    except (sqlite3.OperationalError, sqlite3.IntegrityError) as e:
+    except (OperationalError, sqlite3.IntegrityError) as e:
         raise click.ClickException(str(e))
 
 
@@ -1426,9 +1453,24 @@ def create_database(path, enable_wal, init_spatialite, load_extension):
     is_flag=True,
     help="If table already exists, replace it",
 )
+@click.option(
+    "--transform",
+    is_flag=True,
+    help="If table already exists, try to transform the schema",
+)
 @load_extension_option
 def create_table(
-    path, table, columns, pk, not_null, default, fk, ignore, replace, load_extension
+    path,
+    table,
+    columns,
+    pk,
+    not_null,
+    default,
+    fk,
+    ignore,
+    replace,
+    transform,
+    load_extension,
 ):
     """
     Add a table with the specified columns. Columns should be specified using
@@ -1463,6 +1505,8 @@ def create_table(
             return
         elif replace:
             db[table].drop()
+        elif transform:
+            pass
         else:
             raise click.ClickException(
                 'Table "{}" already exists. Use --replace to delete and replace it.'.format(
@@ -1470,10 +1514,38 @@ def create_table(
                 )
             )
     db[table].create(
-        coltypes, pk=pk, not_null=not_null, defaults=dict(default), foreign_keys=fk
+        coltypes,
+        pk=pk,
+        not_null=not_null,
+        defaults=dict(default),
+        foreign_keys=fk,
+        transform=transform,
     )
 
 
+@cli.command(name="duplicate")
+@click.argument(
+    "path",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@click.argument("table")
+@click.argument("new_table")
+@click.option("--ignore", is_flag=True, help="If table does not exist, do nothing")
+@load_extension_option
+def duplicate(path, table, new_table, ignore, load_extension):
+    """
+    Create a duplicate of this table, copying across the schema and all row data.
+    """
+    db = sqlite_utils.Database(path)
+    _load_extensions(db, load_extension)
+    try:
+        db[table].duplicate(new_table)
+    except NoTable:
+        if not ignore:
+            raise click.ClickException('Table "{}" does not exist'.format(table))
+
+
 @cli.command(name="drop-table")
 @click.argument(
     "path",
@@ -1481,7 +1553,7 @@ def create_table(
     required=True,
 )
 @click.argument("table")
-@click.option("--ignore", is_flag=True)
+@click.option("--ignore", is_flag=True, help="If table does not exist, do nothing")
 @load_extension_option
 def drop_table(path, table, ignore, load_extension):
     """Drop the specified table
@@ -1495,7 +1567,7 @@ def drop_table(path, table, ignore, load_extension):
     _load_extensions(db, load_extension)
     try:
         db[table].drop(ignore=ignore)
-    except sqlite3.OperationalError:
+    except OperationalError:
         raise click.ClickException('Table "{}" does not exist'.format(table))
 
 
@@ -1551,7 +1623,7 @@ def create_view(path, view, select, ignore, replace, load_extension):
     required=True,
 )
 @click.argument("view")
-@click.option("--ignore", is_flag=True)
+@click.option("--ignore", is_flag=True, help="If view does not exist, do nothing")
 @load_extension_option
 def drop_view(path, view, ignore, load_extension):
     """Drop the specified view
@@ -1565,7 +1637,7 @@ def drop_view(path, view, ignore, load_extension):
     _load_extensions(db, load_extension)
     try:
         db[view].drop(ignore=ignore)
-    except sqlite3.OperationalError:
+    except OperationalError:
         raise click.ClickException('View "{}" does not exist'.format(view))
 
 
@@ -1591,6 +1663,9 @@ def drop_view(path, view, ignore, load_extension):
     type=(str, str),
     help="Named :parameters for SQL query",
 )
+@click.option(
+    "--functions", help="Python code defining one or more custom SQL functions"
+)
 @load_extension_option
 def query(
     path,
@@ -1607,6 +1682,7 @@ def query(
     raw,
     param,
     load_extension,
+    functions,
 ):
     """Execute SQL query and return the results as JSON
 
@@ -1623,6 +1699,9 @@ def query(
     _load_extensions(db, load_extension)
     db.register_fts4_bm25()
 
+    if functions:
+        _register_functions(db, functions)
+
     _execute_query(
         db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
     )
@@ -1636,6 +1715,9 @@ def query(
     nargs=-1,
 )
 @click.argument("sql")
+@click.option(
+    "--functions", help="Python code defining one or more custom SQL functions"
+)
 @click.option(
     "--attach",
     type=(str, click.Path(file_okay=True, dir_okay=False, allow_dash=False)),
@@ -1682,6 +1764,7 @@ def query(
 def memory(
     paths,
     sql,
+    functions,
     attach,
     flatten,
     nl,
@@ -1795,6 +1878,9 @@ def memory(
     _load_extensions(db, load_extension)
     db.register_fts4_bm25()
 
+    if functions:
+        _register_functions(db, functions)
+
     _execute_query(
         db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
     )
@@ -1806,7 +1892,7 @@ def _execute_query(
     with db.conn:
         try:
             cursor = db.execute(sql, dict(param))
-        except sqlite3.OperationalError as e:
+        except OperationalError as e:
             raise click.ClickException(str(e))
         if cursor.description is None:
             # This was an update/insert
@@ -1943,6 +2029,7 @@ def search(
 @click.argument("dbtable")
 @click.option("-c", "--column", type=str, multiple=True, help="Columns to return")
 @click.option("--where", help="Optional where clause")
+@click.option("-o", "--order", type=str, help="Order by ('column' or 'column desc')")
 @click.option(
     "-p",
     "--param",
@@ -1969,6 +2056,7 @@ def rows(
     dbtable,
     column,
     where,
+    order,
     param,
     limit,
     offset,
@@ -1995,6 +2083,8 @@ def rows(
     sql = "select {} from [{}]".format(columns, dbtable)
     if where:
         sql += " where " + where
+    if order:
+        sql += " order by " + order
     if limit:
         sql += " limit {}".format(limit)
     if offset:
@@ -2929,4 +3019,22 @@ def _load_extensions(db, load_extension):
         for ext in load_extension:
             if ext == "spatialite" and not os.path.exists(ext):
                 ext = find_spatialite()
-            db.conn.load_extension(ext)
+            if ":" in ext:
+                path, _, entrypoint = ext.partition(":")
+                db.conn.execute("SELECT load_extension(?, ?)", [path, entrypoint])
+            else:
+                db.conn.load_extension(ext)
+
+
+def _register_functions(db, functions):
+    # Register any Python functions as SQL functions:
+    sqlite3.enable_callback_tracebacks(True)
+    globals = {}
+    try:
+        exec(functions, globals)
+    except SyntaxError as ex:
+        raise click.ClickException("Error in functions definition: {}".format(ex))
+    # Register all callables in the locals dict:
+    for name, value in globals.items():
+        if callable(value) and not name.startswith("_"):
+            db.register_function(value, name=name)
