@@ -1,6 +1,7 @@
 from sqlite_utils import cli, Database
 from sqlite_utils.db import Index, ForeignKey
 from click.testing import CliRunner
+from pathlib import Path
 import subprocess
 import sys
 from unittest import mock
@@ -12,6 +13,26 @@ import textwrap
 from .utils import collapse_whitespace
 
 
+def _supports_pragma_function_list():
+    db = Database(memory=True)
+    try:
+        db.execute("select * from pragma_function_list()")
+    except Exception:
+        return False
+    return True
+
+
+def _has_compiled_ext():
+    for ext in ["dylib", "so", "dll"]:
+        path = Path(__file__).parent / f"ext.{ext}"
+        if path.is_file():
+            return True
+    return False
+
+
+COMPILED_EXTENSION_PATH = str(Path(__file__).parent / "ext")
+
+
 @pytest.mark.parametrize(
     "options",
     (
@@ -214,13 +235,12 @@ def test_create_index(db_path):
     ] == db["Gosh2"].indexes
     # Trying to create the same index should fail
     assert 0 != CliRunner().invoke(cli.cli, create_index_unique_args).exit_code
-    # ... unless we use --if-not-exists
-    assert (
-        0
-        == CliRunner()
-        .invoke(cli.cli, create_index_unique_args + ["--if-not-exists"])
-        .exit_code
-    )
+    # ... unless we use --if-not-exists or --ignore
+    for option in ("--if-not-exists", "--ignore"):
+        assert (
+            CliRunner().invoke(cli.cli, create_index_unique_args + [option]).exit_code
+            == 0
+        )
 
 
 def test_create_index_analyze(db_path):
@@ -272,6 +292,19 @@ def test_add_column(db_path, col_name, col_type, expected_schema):
     assert expected_schema == collapse_whitespace(db["dogs"].schema)
 
 
+@pytest.mark.parametrize("ignore", (True, False))
+def test_add_column_ignore(db_path, ignore):
+    db = Database(db_path)
+    db.create_table("dogs", {"name": str})
+    args = ["add-column", db_path, "dogs", "name"] + (["--ignore"] if ignore else [])
+    result = CliRunner().invoke(cli.cli, args)
+    if ignore:
+        assert result.exit_code == 0
+    else:
+        assert result.exit_code == 1
+        assert result.output == "Error: duplicate column name: name\n"
+
+
 def test_add_column_not_null_default(db_path):
     db = Database(db_path)
     db.create_table("dogs", {"name": str})
@@ -455,6 +488,31 @@ def test_enable_fts(db_path):
     db["http://example.com"].drop()
 
 
+def test_enable_fts_replace(db_path):
+    db = Database(db_path)
+    assert db["Gosh"].detect_fts() is None
+    result = CliRunner().invoke(
+        cli.cli, ["enable-fts", db_path, "Gosh", "c1", "--fts4"]
+    )
+    assert result.exit_code == 0
+    assert "Gosh_fts" == db["Gosh"].detect_fts()
+    assert db["Gosh_fts"].columns_dict == {"c1": str}
+
+    # This should throw an error
+    result2 = CliRunner().invoke(
+        cli.cli, ["enable-fts", db_path, "Gosh", "c1", "--fts4"]
+    )
+    assert result2.exit_code == 1
+    assert result2.output == "Error: table [Gosh_fts] already exists\n"
+
+    # This should work
+    result3 = CliRunner().invoke(
+        cli.cli, ["enable-fts", db_path, "Gosh", "c2", "--fts4", "--replace"]
+    )
+    assert result3.exit_code == 0
+    assert db["Gosh_fts"].columns_dict == {"c2": str}
+
+
 def test_enable_fts_with_triggers(db_path):
     Database(db_path)["Gosh"].insert_all([{"c1": "baz"}])
     exit_code = (
@@ -646,6 +704,11 @@ _one_query = "select id, name, age from dogs where id = 1"
         (_one_query, ["--nl"], '{"id": 1, "name": "Cleo", "age": 4}'),
         (_one_query, ["--arrays"], '[[1, "Cleo", 4]]'),
         (_one_query, ["--arrays", "--nl"], '[1, "Cleo", 4]'),
+        (
+            "select id, dog(age) from dogs",
+            ["--functions", "def dog(i):\n  return i * 7"],
+            '[{"id": 1, "dog(age)": 28},\n {"id": 2, "dog(age)": 14}]',
+        ),
     ],
 )
 def test_query_json(db_path, sql, args, expected):
@@ -663,11 +726,76 @@ def test_query_json(db_path, sql, args, expected):
 
 def test_query_json_empty(db_path):
     result = CliRunner().invoke(
-        cli.cli, [db_path, "select * from sqlite_master where 0"]
+        cli.cli,
+        [db_path, "select * from sqlite_master where 0"],
     )
     assert result.output.strip() == "[]"
 
 
+def test_query_invalid_function(db_path):
+    result = CliRunner().invoke(
+        cli.cli, [db_path, "select bad()", "--functions", "def invalid_python"]
+    )
+    assert result.exit_code == 1
+    assert (
+        result.output.strip()
+        == "Error: Error in functions definition: invalid syntax (<string>, line 1)"
+    )
+
+
+TEST_FUNCTIONS = """
+def zero():
+    return 0
+
+def one(a):
+    return a
+
+def _two(a, b):
+    return a + b
+
+def two(a, b):
+    return _two(a, b)
+"""
+
+
+def test_query_complex_function(db_path):
+    result = CliRunner().invoke(
+        cli.cli,
+        [
+            db_path,
+            "select zero(), one(1), two(1, 2)",
+            "--functions",
+            TEST_FUNCTIONS,
+        ],
+    )
+    assert result.exit_code == 0
+    assert json.loads(result.output.strip()) == [
+        {"zero()": 0, "one(1)": 1, "two(1, 2)": 3}
+    ]
+
+
+@pytest.mark.skipif(
+    not _supports_pragma_function_list(),
+    reason="Needs SQLite version that supports pragma_function_list()",
+)
+def test_hidden_functions_are_hidden(db_path):
+    result = CliRunner().invoke(
+        cli.cli,
+        [
+            db_path,
+            "select name from pragma_function_list()",
+            "--functions",
+            TEST_FUNCTIONS,
+        ],
+    )
+    assert result.exit_code == 0
+    functions = {r["name"] for r in json.loads(result.output.strip())}
+    assert "zero" in functions
+    assert "one" in functions
+    assert "two" in functions
+    assert "_two" not in functions
+
+
 LOREM_IPSUM_COMPRESSED = (
     b"x\x9c\xed\xd1\xcdq\x03!\x0c\x05\xe0\xbb\xabP\x01\x1eW\x91\xdc|M\x01\n\xc8\x8e"
     b"f\xf83H\x1e\x97\x1f\x91M\x8e\xe9\xe0\xdd\x96\x05\x84\xf4\xbek\x9fRI\xc7\xf2J"
@@ -844,6 +972,15 @@ def test_query_memory_does_not_create_file(tmpdir):
             ["-c", "name", "--where", "id = :id", "--param", "id", "1"],
             '[{"name": "Cleo"}]',
         ),
+        # --order
+        (
+            ["-c", "id", "--order", "id desc", "--limit", "1"],
+            '[{"id": 2}]',
+        ),
+        (
+            ["-c", "id", "--order", "id", "--limit", "1"],
+            '[{"id": 1}]',
+        ),
     ],
 )
 def test_rows(db_path, args, expected):
@@ -2129,3 +2266,62 @@ def test_analyze(tmpdir, options, expected):
     result = CliRunner().invoke(cli.cli, ["analyze", db_path] + options)
     assert result.exit_code == 0
     assert list(db["sqlite_stat1"].rows) == expected
+
+
+def test_duplicate_table(tmpdir):
+    db_path = str(tmpdir / "test.db")
+    db = Database(db_path)
+    db["one"].insert({"id": 1, "name": "Cleo"}, pk="id")
+    # First try a non-existent table
+    result_error = CliRunner().invoke(
+        cli.cli,
+        ["duplicate", db_path, "missing", "two"],
+        catch_exceptions=False,
+    )
+    assert result_error.exit_code == 1
+    assert result_error.output == 'Error: Table "missing" does not exist\n'
+    # And check --ignore works
+    result_error2 = CliRunner().invoke(
+        cli.cli,
+        ["duplicate", db_path, "missing", "two", "--ignore"],
+        catch_exceptions=False,
+    )
+    assert result_error2.exit_code == 0
+    # Now try for a table that exists
+    result = CliRunner().invoke(
+        cli.cli,
+        ["duplicate", db_path, "one", "two"],
+        catch_exceptions=False,
+    )
+    assert result.exit_code == 0
+    assert db["one"].columns_dict == db["two"].columns_dict
+    assert list(db["one"].rows) == list(db["two"].rows)
+
+
+@pytest.mark.skipif(not _has_compiled_ext(), reason="Requires compiled ext.c")
+@pytest.mark.parametrize(
+    "entrypoint,should_pass,should_fail",
+    (
+        (None, ("a",), ("b", "c")),
+        ("sqlite3_ext_b_init", ("b"), ("a", "c")),
+        ("sqlite3_ext_c_init", ("c"), ("a", "b")),
+    ),
+)
+def test_load_extension(entrypoint, should_pass, should_fail):
+    ext = COMPILED_EXTENSION_PATH
+    if entrypoint:
+        ext += ":" + entrypoint
+    for func in should_pass:
+        result = CliRunner().invoke(
+            cli.cli,
+            ["memory", "select {}()".format(func), "--load-extension", ext],
+            catch_exceptions=False,
+        )
+        assert result.exit_code == 0
+    for func in should_fail:
+        result = CliRunner().invoke(
+            cli.cli,
+            ["memory", "select {}()".format(func), "--load-extension", ext],
+            catch_exceptions=False,
+        )
+        assert result.exit_code == 1
