@@ -94,6 +94,38 @@ def test_search_limit_offset(fresh_db):
     )
 
 
+@pytest.mark.parametrize("fts_version", ("FTS4", "FTS5"))
+def test_search_where(fresh_db, fts_version):
+    table = fresh_db["t"]
+    table.insert_all(search_records)
+    table.enable_fts(["text", "country"], fts_version=fts_version)
+    results = list(
+        table.search("are", where="country = :country", where_args={"country": "Japan"})
+    )
+    assert results == [
+        {
+            "rowid": 1,
+            "text": "tanuki are running tricksters",
+            "country": "Japan",
+            "not_searchable": "foo",
+        }
+    ]
+
+
+def test_search_where_args_disallows_query(fresh_db):
+    table = fresh_db["t"]
+    with pytest.raises(ValueError) as ex:
+        list(
+            table.search(
+                "x", where="author = :query", where_args={"query": "not allowed"}
+            )
+        )
+    assert (
+        ex.value.args[0]
+        == "'query' is a reserved key and cannot be passed to where_args for .search()"
+    )
+
+
 def test_enable_fts_table_names_containing_spaces(fresh_db):
     table = fresh_db["test"]
     table.insert({"column with spaces": "in its name"})
@@ -148,32 +180,32 @@ def test_populate_fts_escape_table_names(fresh_db):
     ] == list(table.search("usa"))
 
 
-def test_fts_tokenize(fresh_db):
-    for fts_version in ("4", "5"):
-        table_name = "searchable_{}".format(fts_version)
-        table = fresh_db[table_name]
-        table.insert_all(search_records)
-        # Test without porter stemming
-        table.enable_fts(
-            ["text", "country"],
-            fts_version="FTS{}".format(fts_version),
-        )
-        assert [] == list(table.search("bite"))
-        # Test WITH stemming
-        table.disable_fts()
-        table.enable_fts(
-            ["text", "country"],
-            fts_version="FTS{}".format(fts_version),
-            tokenize="porter",
-        )
-        rows = list(table.search("bite", order_by="rowid"))
-        assert len(rows) == 1
-        assert {
-            "rowid": 2,
-            "text": "racoons are biting trash pandas",
-            "country": "USA",
-            "not_searchable": "bar",
-        }.items() <= rows[0].items()
+@pytest.mark.parametrize("fts_version", ("4", "5"))
+def test_fts_tokenize(fresh_db, fts_version):
+    table_name = "searchable_{}".format(fts_version)
+    table = fresh_db[table_name]
+    table.insert_all(search_records)
+    # Test without porter stemming
+    table.enable_fts(
+        ["text", "country"],
+        fts_version="FTS{}".format(fts_version),
+    )
+    assert [] == list(table.search("bite"))
+    # Test WITH stemming
+    table.disable_fts()
+    table.enable_fts(
+        ["text", "country"],
+        fts_version="FTS{}".format(fts_version),
+        tokenize="porter",
+    )
+    rows = list(table.search("bite", order_by="rowid"))
+    assert len(rows) == 1
+    assert {
+        "rowid": 2,
+        "text": "racoons are biting trash pandas",
+        "country": "USA",
+        "not_searchable": "bar",
+    }.items() <= rows[0].items()
 
 
 def test_optimize_fts(fresh_db):
@@ -415,6 +447,28 @@ def test_enable_fts_error_message_on_views():
                 "limit 10"
             ),
         ),
+        (
+            {"where": "author = :author"},
+            "FTS5",
+            (
+                "with original as (\n"
+                "    select\n"
+                "        rowid,\n"
+                "        *\n"
+                "    from [books]\n"
+                "    where author = :author\n"
+                ")\n"
+                "select\n"
+                "    [original].*\n"
+                "from\n"
+                "    [original]\n"
+                "    join [books_fts] on [original].rowid = [books_fts].rowid\n"
+                "where\n"
+                "    [books_fts] match :query\n"
+                "order by\n"
+                "    [books_fts].rank"
+            ),
+        ),
         (
             {"columns": ["title"]},
             "FTS4",
@@ -480,6 +534,28 @@ def test_enable_fts_error_message_on_views():
                 "limit 2"
             ),
         ),
+        (
+            {"where": "author = :author"},
+            "FTS4",
+            (
+                "with original as (\n"
+                "    select\n"
+                "        rowid,\n"
+                "        *\n"
+                "    from [books]\n"
+                "    where author = :author\n"
+                ")\n"
+                "select\n"
+                "    [original].*\n"
+                "from\n"
+                "    [original]\n"
+                "    join [books_fts] on [original].rowid = [books_fts].rowid\n"
+                "where\n"
+                "    [books_fts] match :query\n"
+                "order by\n"
+                "    rank_bm25(matchinfo([books_fts], 'pcnalx'))"
+            ),
+        ),
     ],
 )
 def test_search_sql(kwargs, fts, expected):
